@@ -34,16 +34,18 @@
 #include <errno.h>
 
 #include <glib/gi18n-lib.h>
-#include <gnome-keyring-memory.h>
 #include <nm-setting-connection.h>
 #include <nm-setting-8021x.h>
 
 #include "auth-helpers.h"
 #include "nm-openvpn.h"
 #include "src/nm-openvpn-service.h"
-#include "common-gnome/keyring-helpers.h"
 #include "common/utils.h"
 
+#define PW_TYPE_SAVE   0
+#define PW_TYPE_ASK    1
+#define PW_TYPE_UNUSED 2
+
 static void
 show_password (GtkToggleButton *togglebutton, GtkEntry *password_entry)
 {
@@ -51,17 +53,15 @@ show_password (GtkToggleButton *togglebutton, GtkEntry *password_entry)
 }
 
 static GtkWidget *
-fill_password (GtkBuilder *builder,
-               const char *widget_name,
-               NMConnection *connection,
-               gboolean priv_key_password)
+setup_secret_widget (GtkBuilder *builder,
+                     const char *widget_name,
+                     NMSettingVPN *s_vpn,
+                     const char *secret_key)
 {
-	NMSettingVPN *s_vpn;
+	NMSettingSecretFlags pw_flags = NM_SETTING_SECRET_FLAG_NONE;
 	GtkWidget *widget;
 	GtkWidget *show_passwords;
 	const char *tmp;
-	char *keyring_pw;
-	gboolean unused;
 
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, widget_name));
 	g_assert (widget);
@@ -69,63 +69,18 @@ fill_password (GtkBuilder *builder,
 	show_passwords = GTK_WIDGET (gtk_builder_get_object (builder, "show_passwords"));
 	g_signal_connect (show_passwords, "toggled", G_CALLBACK (show_password), widget);
 
-	if (!connection)
-		return widget;
-
-	/* Grab from the connection first */
-	s_vpn = (NMSettingVPN *) nm_connection_get_setting (connection, NM_TYPE_SETTING_VPN);
 	if (s_vpn) {
-		tmp = nm_setting_vpn_get_secret (s_vpn, priv_key_password ? NM_OPENVPN_KEY_CERTPASS : NM_OPENVPN_KEY_PASSWORD);
-		if (tmp) {
+		tmp = nm_setting_vpn_get_secret (s_vpn, secret_key);
+		if (tmp)
 			gtk_entry_set_text (GTK_ENTRY (widget), tmp);
-			return widget;
-		}
-	}
 
-	/* If not the connection then from the keyring */
-	keyring_pw = keyring_helpers_lookup_secret (nm_connection_get_uuid (connection),
-	                                            priv_key_password ? NM_OPENVPN_KEY_CERTPASS : NM_OPENVPN_KEY_PASSWORD,
-	                                            &unused);
-	if (keyring_pw) {
-		gtk_entry_set_text (GTK_ENTRY (widget), keyring_pw);
-		gnome_keyring_memory_free (keyring_pw);
+		nm_setting_get_secret_flags (NM_SETTING (s_vpn), secret_key, &pw_flags, NULL);
+		g_object_set_data (G_OBJECT (widget), "flags", GUINT_TO_POINTER (pw_flags));
 	}
 
 	return widget;
 }
 
-void
-fill_vpn_passwords (GtkBuilder *builder,
-                    GtkSizeGroup *group,
-                    NMConnection *connection,
-                    const char *contype,
-                    ChangedCallback changed_cb,
-                    gpointer user_data)
-{
-	GtkWidget *w = NULL;
-
-	if (!strcmp (contype, NM_OPENVPN_CONTYPE_TLS))
-		w = fill_password (builder, "tls_private_key_password_entry", connection, TRUE);
-	else if (!strcmp (contype, NM_OPENVPN_CONTYPE_PASSWORD))
-		w = fill_password (builder, "pw_password_entry", connection, FALSE);
-	else if (!strcmp (contype, NM_OPENVPN_CONTYPE_PASSWORD_TLS)) {
-		GtkWidget *w2 = NULL;
-
-		w = fill_password (builder, "pw_tls_password_entry", connection, FALSE);
-	
-		w2 = fill_password (builder, "pw_tls_private_key_password_entry", connection, TRUE);
-		if (w2) {
-			gtk_size_group_add_widget (group, w2);
-			g_signal_connect (w2, "changed", G_CALLBACK (changed_cb), user_data);
-		}
-	}
-
-	if (w) {
-		gtk_size_group_add_widget (group, w);
-		g_signal_connect (w, "changed", G_CALLBACK (changed_cb), user_data);
-	}
-}
-
 static void
 tls_cert_changed_cb (GtkWidget *widget, GtkWidget *next_widget)
 {
@@ -167,6 +122,184 @@ tls_cert_changed_cb (GtkWidget *widget, GtkWidget *next_widget)
 	g_free (next_fname);
 }
 
+static void
+tls_setup (GtkBuilder *builder,
+           GtkSizeGroup *group,
+           NMSettingVPN *s_vpn,
+           const char *prefix,
+           GtkWidget *ca_chooser,
+           ChangedCallback changed_cb,
+           gpointer user_data)
+{
+	GtkWidget *widget, *cert, *key;
+	const char *value;
+	char *tmp;
+	GtkFileFilter *filter;
+
+	tmp = g_strdup_printf ("%s_user_cert_chooser", prefix);
+	cert = GTK_WIDGET (gtk_builder_get_object (builder, tmp));
+	g_free (tmp);
+
+	gtk_size_group_add_widget (group, cert);
+	filter = tls_file_chooser_filter_new (TRUE);
+	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (cert), filter);
+	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (cert), TRUE);
+	gtk_file_chooser_button_set_title (GTK_FILE_CHOOSER_BUTTON (cert),
+	                                   _("Choose your personal certificate..."));
+	g_signal_connect (G_OBJECT (cert), "selection-changed", G_CALLBACK (changed_cb), user_data);
+
+	if (s_vpn) {
+		value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CERT);
+		if (value && strlen (value))
+			gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (cert), value);
+	}
+
+	tmp = g_strdup_printf ("%s_private_key_chooser", prefix);
+	key = GTK_WIDGET (gtk_builder_get_object (builder, tmp));
+	g_free (tmp);
+
+	gtk_size_group_add_widget (group, key);
+	filter = tls_file_chooser_filter_new (TRUE);
+	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (key), filter);
+	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (key), TRUE);
+	gtk_file_chooser_button_set_title (GTK_FILE_CHOOSER_BUTTON (key),
+	                                   _("Choose your private key..."));
+	g_signal_connect (G_OBJECT (key), "selection-changed", G_CALLBACK (changed_cb), user_data);
+
+	if (s_vpn) {
+		value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_KEY);
+		if (value && strlen (value))
+			gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (key), value);
+	}
+
+	/* Link choosers to the PKCS#12 changer callback */
+	g_signal_connect (ca_chooser, "selection-changed", G_CALLBACK (tls_cert_changed_cb), cert);
+	g_signal_connect (cert, "selection-changed", G_CALLBACK (tls_cert_changed_cb), key);
+	g_signal_connect (key, "selection-changed", G_CALLBACK (tls_cert_changed_cb), ca_chooser);
+
+	/* Fill in the private key password */
+	tmp = g_strdup_printf ("%s_private_key_password_entry", prefix);
+	widget = setup_secret_widget (builder, tmp, s_vpn, NM_OPENVPN_KEY_CERTPASS);
+	g_free (tmp);
+	gtk_size_group_add_widget (group, widget);
+	g_signal_connect (widget, "changed", G_CALLBACK (changed_cb), user_data);
+}
+
+static void
+pw_type_combo_changed_cb (GtkWidget *combo, gpointer user_data)
+{
+	GtkWidget *entry = user_data;
+
+	/* If the user chose "Not required", desensitize and clear the correct
+	 * password entry.
+	 */
+	switch (gtk_combo_box_get_active (GTK_COMBO_BOX (combo))) {
+	case PW_TYPE_ASK:
+	case PW_TYPE_UNUSED:
+		gtk_entry_set_text (GTK_ENTRY (entry), "");
+		gtk_widget_set_sensitive (entry, FALSE);
+		break;
+	default:
+		gtk_widget_set_sensitive (entry, TRUE);
+		break;
+	}
+}
+
+static void
+init_one_pw_combo (GtkBuilder *builder,
+                   NMSettingVPN *s_vpn,
+                   const char *prefix,
+                   const char *secret_key,
+                   GtkWidget *entry_widget,
+                   ChangedCallback changed_cb,
+                   gpointer user_data)
+{
+	int active = -1;
+	GtkWidget *widget;
+	GtkListStore *store;
+	GtkTreeIter iter;
+	const char *value = NULL;
+	char *tmp;
+	guint32 default_idx = 1;
+	NMSettingSecretFlags pw_flags = NM_SETTING_SECRET_FLAG_NONE;
+
+	/* If there's already a password and the password type can't be found in
+	 * the VPN settings, default to saving it.  Otherwise, always ask for it.
+	 */
+	value = gtk_entry_get_text (GTK_ENTRY (entry_widget));
+	if (value && strlen (value))
+		default_idx = 0;
+
+	store = gtk_list_store_new (1, G_TYPE_STRING);
+	if (s_vpn)
+		nm_setting_get_secret_flags (NM_SETTING (s_vpn), secret_key, &pw_flags, NULL);
+
+	gtk_list_store_append (store, &iter);
+	gtk_list_store_set (store, &iter, 0, _("Saved"), -1);
+	if (   (active < 0)
+	    && !(pw_flags & NM_SETTING_SECRET_FLAG_NOT_SAVED)
+	    && !(pw_flags & NM_SETTING_SECRET_FLAG_NOT_REQUIRED)) {
+		active = PW_TYPE_SAVE;
+	}
+
+	gtk_list_store_append (store, &iter);
+	gtk_list_store_set (store, &iter, 0, _("Always Ask"), -1);
+	if ((active < 0) && (pw_flags & NM_SETTING_SECRET_FLAG_NOT_SAVED))
+		active = PW_TYPE_ASK;
+
+	gtk_list_store_append (store, &iter);
+	gtk_list_store_set (store, &iter, 0, _("Not Required"), -1);
+	if ((active < 0) && (pw_flags & NM_SETTING_SECRET_FLAG_NOT_REQUIRED))
+		active = PW_TYPE_UNUSED;
+
+	tmp = g_strdup_printf ("%s_pass_type_combo", prefix);
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, tmp));
+	g_assert (widget);
+	g_free (tmp);
+
+	gtk_combo_box_set_model (GTK_COMBO_BOX (widget), GTK_TREE_MODEL (store));
+	g_object_unref (store);
+	gtk_combo_box_set_active (GTK_COMBO_BOX (widget), active < 0 ? default_idx : active);
+	pw_type_combo_changed_cb (widget, entry_widget);
+
+	g_signal_connect (G_OBJECT (widget), "changed", G_CALLBACK (pw_type_combo_changed_cb), entry_widget);
+	g_signal_connect (G_OBJECT (widget), "changed", G_CALLBACK (changed_cb), user_data);
+}
+
+static void
+pw_setup (GtkBuilder *builder,
+          GtkSizeGroup *group, 
+          NMSettingVPN *s_vpn,
+          const char *prefix,
+          ChangedCallback changed_cb,
+          gpointer user_data)
+{
+	GtkWidget *widget;
+	const char *value;
+	char *tmp;
+
+	tmp = g_strdup_printf ("%s_username_entry", prefix);
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, tmp));
+	g_free (tmp);
+	gtk_size_group_add_widget (group, widget);
+
+	if (s_vpn) {
+		value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_USERNAME);
+		if (value && strlen (value))
+			gtk_entry_set_text (GTK_ENTRY (widget), value);
+	}
+	g_signal_connect (G_OBJECT (widget), "changed", G_CALLBACK (changed_cb), user_data);
+
+	/* Fill in the user password */
+	tmp = g_strdup_printf ("%s_password_entry", prefix);
+	widget = setup_secret_widget (builder, tmp, s_vpn, NM_OPENVPN_KEY_PASSWORD);
+	g_free (tmp);
+	gtk_size_group_add_widget (group, widget);
+	g_signal_connect (widget, "changed", G_CALLBACK (changed_cb), user_data);
+
+	init_one_pw_combo (builder, s_vpn, prefix, NM_OPENVPN_KEY_PASSWORD, widget, changed_cb, user_data);
+}
+
 void
 tls_pw_init_auth_widget (GtkBuilder *builder,
                          GtkSizeGroup *group,
@@ -176,10 +309,11 @@ tls_pw_init_auth_widget (GtkBuilder *builder,
                          ChangedCallback changed_cb,
                          gpointer user_data)
 {
-	GtkWidget *widget, *ca, *cert, *key;
+	GtkWidget *ca;
 	const char *value;
 	char *tmp;
 	GtkFileFilter *filter;
+	gboolean tls = FALSE, pw = FALSE;
 
 	g_return_if_fail (builder != NULL);
 	g_return_if_fail (group != NULL);
@@ -189,13 +323,18 @@ tls_pw_init_auth_widget (GtkBuilder *builder,
 	tmp = g_strdup_printf ("%s_ca_cert_chooser", prefix);
 	ca = GTK_WIDGET (gtk_builder_get_object (builder, tmp));
 	g_free (tmp);
-
 	gtk_size_group_add_widget (group, ca);
-	if (!strcmp (contype, NM_OPENVPN_CONTYPE_TLS) || !strcmp (contype, NM_OPENVPN_CONTYPE_PASSWORD_TLS))
-		filter = tls_file_chooser_filter_new (TRUE);
-	else
-		filter = tls_file_chooser_filter_new (FALSE);
 
+	/* Three major connection types here: TLS-only, PW-only, and TLS + PW */
+	if (!strcmp (contype, NM_OPENVPN_CONTYPE_TLS) || !strcmp (contype, NM_OPENVPN_CONTYPE_PASSWORD_TLS))
+		tls = TRUE;
+	if (!strcmp (contype, NM_OPENVPN_CONTYPE_PASSWORD) || !strcmp (contype, NM_OPENVPN_CONTYPE_PASSWORD_TLS))
+		pw = TRUE;
+
+	/* Only TLS types can use PKCS#12 */
+	filter = tls_file_chooser_filter_new (tls);
+
+	/* Set up CA cert file picker which all connection types support */
 	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (ca), filter);
 	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (ca), TRUE);
 	gtk_file_chooser_button_set_title (GTK_FILE_CHOOSER_BUTTON (ca),
@@ -208,62 +347,11 @@ tls_pw_init_auth_widget (GtkBuilder *builder,
 			gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (ca), value);
 	}
 
-	if (!strcmp (contype, NM_OPENVPN_CONTYPE_TLS) || !strcmp (contype, NM_OPENVPN_CONTYPE_PASSWORD_TLS)) {
-		tmp = g_strdup_printf ("%s_user_cert_chooser", prefix);
-		cert = GTK_WIDGET (gtk_builder_get_object (builder, tmp));
-		g_free (tmp);
-
-		gtk_size_group_add_widget (group, cert);
-		filter = tls_file_chooser_filter_new (TRUE);
-		gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (cert), filter);
-		gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (cert), TRUE);
-		gtk_file_chooser_button_set_title (GTK_FILE_CHOOSER_BUTTON (cert),
-		                                   _("Choose your personal certificate..."));
-		g_signal_connect (G_OBJECT (cert), "selection-changed", G_CALLBACK (changed_cb), user_data);
-
-		if (s_vpn) {
-			value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CERT);
-			if (value && strlen (value))
-				gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (cert), value);
-		}
-
-		tmp = g_strdup_printf ("%s_private_key_chooser", prefix);
-		key = GTK_WIDGET (gtk_builder_get_object (builder, tmp));
-		g_free (tmp);
-
-		gtk_size_group_add_widget (group, key);
-		filter = tls_file_chooser_filter_new (TRUE);
-		gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (key), filter);
-		gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (key), TRUE);
-		gtk_file_chooser_button_set_title (GTK_FILE_CHOOSER_BUTTON (key),
-		                                   _("Choose your private key..."));
-		g_signal_connect (G_OBJECT (key), "selection-changed", G_CALLBACK (changed_cb), user_data);
-
-		if (s_vpn) {
-			value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_KEY);
-			if (value && strlen (value))
-				gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (key), value);
-		}
-
-		/* Link choosers to the PKCS#12 changer callback */
-		g_signal_connect (ca, "selection-changed", G_CALLBACK (tls_cert_changed_cb), cert);
-		g_signal_connect (cert, "selection-changed", G_CALLBACK (tls_cert_changed_cb), key);
-		g_signal_connect (key, "selection-changed", G_CALLBACK (tls_cert_changed_cb), ca);
-	}
-
-	if (!strcmp (contype, NM_OPENVPN_CONTYPE_PASSWORD) || !strcmp (contype, NM_OPENVPN_CONTYPE_PASSWORD_TLS)) {
-		tmp = g_strdup_printf ("%s_username_entry", prefix);
-		widget = GTK_WIDGET (gtk_builder_get_object (builder, tmp));
-		g_free (tmp);
-
-		gtk_size_group_add_widget (group, widget);
-		if (s_vpn) {
-			value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_USERNAME);
-			if (value && strlen (value))
-				gtk_entry_set_text (GTK_ENTRY (widget), value);
-		}
-		g_signal_connect (G_OBJECT (widget), "changed", G_CALLBACK (changed_cb), user_data);
-	}
+	/* Set up the rest of the options */
+	if (tls)
+		tls_setup (builder, group, s_vpn, prefix, ca, changed_cb, user_data);
+	if (pw)
+		pw_setup (builder, group, s_vpn, prefix, changed_cb, user_data);
 }
 
 #define SK_DIR_COL_NAME 0
@@ -539,15 +627,34 @@ update_from_filechooser (GtkBuilder *builder,
 static void
 update_tls (GtkBuilder *builder, const char *prefix, NMSettingVPN *s_vpn)
 {
+	GtkWidget *widget;
+	NMSettingSecretFlags pw_flags;
+	char *tmp;
+	const char *str;
+
 	update_from_filechooser (builder, NM_OPENVPN_KEY_CA, prefix, "ca_cert_chooser", s_vpn);
 	update_from_filechooser (builder, NM_OPENVPN_KEY_CERT, prefix, "user_cert_chooser", s_vpn);
 	update_from_filechooser (builder, NM_OPENVPN_KEY_KEY, prefix, "private_key_chooser", s_vpn);
+
+	/* Password */
+	tmp = g_strdup_printf ("%s_private_key_password_entry", prefix);
+	widget = (GtkWidget *) gtk_builder_get_object (builder, tmp);
+	g_assert (widget);
+	g_free (tmp);
+
+	str = gtk_entry_get_text (GTK_ENTRY (widget));
+	if (str && strlen (str))
+		nm_setting_vpn_add_secret (s_vpn, NM_OPENVPN_KEY_CERTPASS, str);
+
+	pw_flags = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (widget), "flags"));
+	nm_setting_set_secret_flags (NM_SETTING (s_vpn), NM_OPENVPN_KEY_CERTPASS, pw_flags, NULL);
 }
 
 static void
-update_username (GtkBuilder *builder, const char *prefix, NMSettingVPN *s_vpn)
+update_pw (GtkBuilder *builder, const char *prefix, NMSettingVPN *s_vpn)
 {
 	GtkWidget *widget;
+	NMSettingSecretFlags pw_flags;
 	char *tmp;
 	const char *str;
 
@@ -562,6 +669,38 @@ update_username (GtkBuilder *builder, const char *prefix, NMSettingVPN *s_vpn)
 	str = gtk_entry_get_text (GTK_ENTRY (widget));
 	if (str && strlen (str))
 		nm_setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_USERNAME, str);
+
+	/* Password */
+	tmp = g_strdup_printf ("%s_password_entry", prefix);
+	widget = (GtkWidget *) gtk_builder_get_object (builder, tmp);
+	g_assert (widget);
+	g_free (tmp);
+
+	str = gtk_entry_get_text (GTK_ENTRY (widget));
+	if (str && strlen (str))
+		nm_setting_vpn_add_secret (s_vpn, NM_OPENVPN_KEY_PASSWORD, str);
+
+	/* Update password flags */
+	pw_flags = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (widget), "flags"));
+	pw_flags &= ~(NM_SETTING_SECRET_FLAG_NOT_SAVED | NM_SETTING_SECRET_FLAG_NOT_REQUIRED);
+
+	tmp = g_strdup_printf ("%s_pass_type_combo", prefix);
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, tmp));
+	g_free (tmp);
+
+	switch (gtk_combo_box_get_active (GTK_COMBO_BOX (widget))) {
+	case PW_TYPE_SAVE:
+		break;
+	case PW_TYPE_UNUSED:
+		pw_flags |= NM_SETTING_SECRET_FLAG_NOT_REQUIRED;
+		break;
+	case PW_TYPE_ASK:
+	default:
+		pw_flags |= NM_SETTING_SECRET_FLAG_NOT_SAVED;
+		break;
+	}
+
+	nm_setting_set_secret_flags (NM_SETTING (s_vpn), NM_OPENVPN_KEY_PASSWORD, pw_flags, NULL);
 }
 
 gboolean
@@ -578,10 +717,10 @@ auth_widget_update_connection (GtkBuilder *builder,
 		update_tls (builder, "tls", s_vpn);
 	} else if (!strcmp (contype, NM_OPENVPN_CONTYPE_PASSWORD)) {
 		update_from_filechooser (builder, NM_OPENVPN_KEY_CA, "pw", "ca_cert_chooser", s_vpn);
-		update_username (builder, "pw", s_vpn);
+		update_pw (builder, "pw", s_vpn);
 	} else if (!strcmp (contype, NM_OPENVPN_CONTYPE_PASSWORD_TLS)) {
 		update_tls (builder, "pw_tls", s_vpn);
-		update_username (builder, "pw_tls", s_vpn);
+		update_pw (builder, "pw_tls", s_vpn);
 	} else if (!strcmp (contype, NM_OPENVPN_CONTYPE_STATIC_KEY)) {
 		/* Update static key */
 		update_from_filechooser (builder, NM_OPENVPN_KEY_STATIC_KEY, "sk", "key_chooser", s_vpn);
@@ -620,81 +759,6 @@ auth_widget_update_connection (GtkBuilder *builder,
 	return TRUE;
 }
 
-static gboolean
-save_secret (GtkBuilder *builder,
-			 const char *widget_name,
-			 const char *vpn_uuid,
-			 const char *vpn_name,
-			 const char *secret_name)
-{
-	GtkWidget *w;
-	const char *secret;
-	GnomeKeyringResult result;
-	gboolean ret;
-
-	w = GTK_WIDGET (gtk_builder_get_object (builder, widget_name));
-	g_assert (w);
-	secret = gtk_entry_get_text (GTK_ENTRY (w));
-	if (secret && strlen (secret)) {
-		result = keyring_helpers_save_secret (vpn_uuid, vpn_name, NULL, secret_name, secret);
-		ret = result == GNOME_KEYRING_RESULT_OK;
-		if (!ret)
-			g_warning ("%s: failed to save user password to keyring.", __func__);
-	} else
-		ret = keyring_helpers_delete_secret (vpn_uuid, secret_name);
-
-	return ret;
-}
-
-gboolean
-auth_widget_save_secrets (GtkBuilder *builder,
-						  const char *contype,
-						  NMConnection *connection)
-{
-	NMSetting *s_vpn;
-	gboolean ret = TRUE;
-	const char *uuid, *name;
-	NMSettingSecretFlags pw_flags = NM_SETTING_SECRET_FLAG_NONE;
-	NMSettingSecretFlags certpw_flags = NM_SETTING_SECRET_FLAG_NONE;
-
-	uuid = nm_connection_get_uuid (connection);
-	name = nm_connection_get_id (connection);
-
-	s_vpn = nm_connection_get_setting (connection, NM_TYPE_SETTING_VPN);
-	if (!s_vpn)
-		return FALSE;
-
-	nm_setting_get_secret_flags (s_vpn, NM_OPENVPN_KEY_PASSWORD, &pw_flags, NULL);
-	nm_setting_get_secret_flags (s_vpn, NM_OPENVPN_KEY_CERTPASS, &certpw_flags, NULL);
-
-	if (!strcmp (contype, NM_OPENVPN_CONTYPE_TLS)) {
-		if (certpw_flags & NM_SETTING_SECRET_FLAG_AGENT_OWNED)
-			ret = save_secret (builder, "tls_private_key_password_entry", uuid, name, NM_OPENVPN_KEY_CERTPASS);
-		else
-			keyring_helpers_delete_secret (uuid, NM_OPENVPN_KEY_CERTPASS);
-	} else if (!strcmp (contype, NM_OPENVPN_CONTYPE_PASSWORD)) {
-		if (pw_flags & NM_SETTING_SECRET_FLAG_AGENT_OWNED)
-			ret = save_secret (builder, "pw_password_entry", uuid, name, NM_OPENVPN_KEY_PASSWORD);
-		else
-			keyring_helpers_delete_secret (uuid, NM_OPENVPN_KEY_PASSWORD);
-	} else if (!strcmp (contype, NM_OPENVPN_CONTYPE_PASSWORD_TLS)) {
-		if (pw_flags & NM_SETTING_SECRET_FLAG_AGENT_OWNED)
-			ret = save_secret (builder, "pw_tls_password_entry", uuid, name, NM_OPENVPN_KEY_PASSWORD);
-		else
-			keyring_helpers_delete_secret (uuid, NM_OPENVPN_KEY_PASSWORD);
-
-		if (certpw_flags & NM_SETTING_SECRET_FLAG_AGENT_OWNED)
-			ret = save_secret (builder, "pw_tls_private_key_password_entry", uuid, name, NM_OPENVPN_KEY_CERTPASS);
-		else
-			keyring_helpers_delete_secret (uuid, NM_OPENVPN_KEY_CERTPASS);
-	} else if (!strcmp (contype, NM_OPENVPN_CONTYPE_STATIC_KEY)) {
-		/* No secrets here */
-	} else
-		g_assert_not_reached ();
-
-	return ret;
-}
-
 static const char *
 find_tag (const char *tag, const char *buf, gsize len)
 {
@@ -1244,6 +1308,8 @@ advanced_dialog_new (GHashTable *hash, const char *contype)
 	ui_file = g_strdup_printf ("%s/%s", UIDIR, "nm-openvpn-dialog.ui");
 	builder = gtk_builder_new ();
 
+	gtk_builder_set_translation_domain (builder, GETTEXT_PACKAGE);
+
 	if (!gtk_builder_add_from_file (builder, ui_file, &error)) {
 		g_warning ("Couldn't load builder file: %s", error->message);
 		g_error_free (error);
@@ -1251,8 +1317,6 @@ advanced_dialog_new (GHashTable *hash, const char *contype)
 		goto out;
 	}
 
-	gtk_builder_set_translation_domain (builder, GETTEXT_PACKAGE);
-
 	dialog = GTK_WIDGET (gtk_builder_get_object (builder, "openvpn-advanced-dialog"));
 	if (!dialog) {
 		g_object_unref (G_OBJECT (builder));
@@ -1330,6 +1394,16 @@ advanced_dialog_new (GHashTable *hash, const char *contype)
 			widget = GTK_WIDGET (gtk_builder_get_object (builder, "proxy_password_entry"));
 			gtk_entry_set_text (GTK_ENTRY (widget), value);
 		}
+
+		value = g_hash_table_lookup (hash, NM_OPENVPN_KEY_HTTP_PROXY_PASSWORD"-flags");
+		if (value && strlen (value)) {
+			errno = 0;
+			tmp = strtol (value, NULL, 10);
+			if (errno != 0 || tmp < 0 || tmp > 65535)
+				tmp = 0;
+			widget = GTK_WIDGET (gtk_builder_get_object (builder, "proxy_password_entry"));
+			g_object_set_data (G_OBJECT (widget), "flags", GUINT_TO_POINTER ((guint32) tmp));
+		}
 	}
 
 	value = g_hash_table_lookup (hash, NM_OPENVPN_KEY_PROXY_TYPE);
@@ -1608,6 +1682,8 @@ advanced_dialog_new_hash_from_dialog (GtkWidget *dialog, GError **error)
 				g_hash_table_insert (hash, g_strdup (NM_OPENVPN_KEY_PROXY_RETRY), g_strdup ("yes"));
 
 			if (proxy_type == PROXY_TYPE_HTTP) {
+				guint32 pw_flags;
+
 				widget = GTK_WIDGET (gtk_builder_get_object (builder, "proxy_username_entry"));
 				value = (char *) gtk_entry_get_text (GTK_ENTRY (widget));
 				if (value && strlen (value)) {
@@ -1623,6 +1699,13 @@ advanced_dialog_new_hash_from_dialog (GtkWidget *dialog, GError **error)
 					                     g_strdup (NM_OPENVPN_KEY_HTTP_PROXY_PASSWORD),
 					                     g_strdup (value));
 				}
+
+				pw_flags = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (widget), "flags"));
+				if (pw_flags != NM_SETTING_SECRET_FLAG_NONE) {
+					g_hash_table_insert (hash,
+					                     g_strdup (NM_OPENVPN_KEY_HTTP_PROXY_PASSWORD"-flags"),
+					                     g_strdup_printf ("%d", pw_flags));
+				}
 			}
 		}
 	}
@@ -1711,34 +1794,3 @@ advanced_dialog_new_hash_from_dialog (GtkWidget *dialog, GError **error)
 	return hash;
 }
 
-gboolean
-advanced_save_secrets (GHashTable *advanced, NMConnection *connection)
-{
-	NMSetting *s_vpn;
-	const char *secret, *uuid, *name;
-	GnomeKeyringResult result;
-	gboolean ret = FALSE;
-	NMSettingSecretFlags flags = NM_SETTING_SECRET_FLAG_NONE;
-
-	s_vpn = nm_connection_get_setting (connection, NM_TYPE_SETTING_VPN);
-	if (!s_vpn)
-		return FALSE;
-	nm_setting_get_secret_flags (s_vpn, NM_OPENVPN_KEY_HTTP_PROXY_PASSWORD, &flags, NULL);
-
-	uuid = nm_connection_get_uuid (connection);
-	name = nm_connection_get_id (connection);
-
-	secret = g_hash_table_lookup (advanced, NM_OPENVPN_KEY_HTTP_PROXY_PASSWORD);
-	if (secret && strlen (secret) && (flags & NM_SETTING_SECRET_FLAG_AGENT_OWNED)) {
-		/* Only save the proxy password if it's agent-owned */
-		result = keyring_helpers_save_secret (uuid, name, NULL, NM_OPENVPN_KEY_HTTP_PROXY_PASSWORD, secret);
-		if (result == GNOME_KEYRING_RESULT_OK)
-			ret = TRUE;
-		else
-			g_warning ("%s: failed to save HTTP proxy password to keyring.", __func__);
-	} else
-		ret = keyring_helpers_delete_secret (uuid, NM_OPENVPN_KEY_HTTP_PROXY_PASSWORD);
-
-	return ret;
-}
-
